@@ -33,8 +33,11 @@ class LogstashFormatter(logging.Formatter):
                  json_cls=None,
                  json_default=_default_json_default):
         """
-        :param source_host: override source host name
-        :param extra: provide extra fields always present in logs
+        :param fmt: Config as a JSON string, allowed fields;
+               extra: provide extra fields always present in logs
+               source_host: override source host name
+        :param datefmt: Date format to use (required by logging.Formatter
+            interface but not used)
         :param json_cls: JSON encoder to forward to json.dumps
         :param json_default: Default JSON representation for unknown types,
                              by default coerce everything to a string
@@ -87,13 +90,33 @@ class LogstashFormatter(logging.Formatter):
             fields.pop('exc_text')
 
         logr = self.defaults.copy()
+
         logr.update({'@message': msg,
                      '@timestamp': datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
                      '@source_host': self.source_host,
-                     '@fields': fields})
+                     '@fields': self._build_fields(logr, fields)})
 
         return json.dumps(logr, default=self.json_default, cls=self.json_cls)
 
+    def _build_fields(self, defaults, fields):
+        """Return provided fields including any in defaults
+
+        >>> f = LogstashFormatter()
+        # Verify that ``fields`` is used
+        >>> f._build_fields({}, {'foo': 'one'}) == \
+                {'foo': 'one'}
+        True
+        # Verify that ``@fields`` in ``defaults`` is used
+        >>> f._build_fields({'@fields': {'bar': 'two'}}, {'foo': 'one'}) == \
+                {'foo': 'one', 'bar': 'two'}
+        True
+        # Verify that ``fields`` takes precedence
+        >>> f._build_fields({'@fields': {'foo': 'two'}}, {'foo': 'one'}) == \
+                {'foo': 'one'}
+        True
+        """
+        return dict(defaults.get('@fields', {}).items() + fields.items())
+    
 
 class LogstashFormatterV1(LogstashFormatter):
     """
@@ -119,7 +142,8 @@ class LogstashFormatterV1(LogstashFormatter):
         if 'exc_text' in fields and not fields['exc_text']:
             fields.pop('exc_text')
 
-        base_log = {'@timestamp': datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
+	now = datetime.datetime.utcnow()
+        base_log = {'@timestamp': now.strftime("%Y-%m-%dT%H:%M:%S") + ".%03d" % (now.microsecond / 1000) + "Z",
                     '@version': 1,
                     'source_host': self.source_host}
         base_log.update(fields)
